@@ -590,8 +590,8 @@ pub(crate) struct Cache {
     pub resolved: crate::Config,
     /// The hex-length to assume when shortening object ids. If `None`, it should be computed based on the approximate object count.
     pub hex_len: Option<usize>,
-    /// true if the repository is designated as 'bare', without work tree.
-    pub is_bare: bool,
+    /// `true` if the repository is designated as 'bare', without work tree. If `None`, the value wasn't configured.
+    pub is_bare: Option<bool>,
     /// The type of hash to use.
     pub object_hash: gix_hash::Kind,
     /// If true, multi-pack indices, whether present or not, may be used by the object database.
