@@ -6,28 +6,247 @@ use std::{
 };
 
 use email_address::{EmailAddress, Options as EmailAddressOptions};
-use fancy_regex::Regex;
 use serde_json::{Map, Value};
-use std::sync::LazyLock;
 use unicode_general_category::{get_general_category, GeneralCategory};
 use uuid_simd::{parse_hyphenated, Out};
 
 use crate::{
-    compiler, ecma,
-    error::ValidationError,
+    compiler,
+    error::{no_error, ValidationError},
+    evaluation::Annotations,
     keywords::CompilationResult,
     paths::{LazyLocation, Location, RefTracker},
     types::JsonType,
-    validator::{Validate, ValidationContext},
+    validator::{EvaluationResult, Validate, ValidationContext},
     Draft,
 };
 
-static URI_TEMPLATE_RE: LazyLock<Regex> = LazyLock::new(|| {
-    Regex::new(
-        r#"^(?:(?:[^\x00-\x20"'<>%\\^`{|}]|%[0-9a-f]{2})|\{[+#./;?&=,!@|]?(?:[a-z0-9_]|%[0-9a-f]{2})+(?::[1-9][0-9]{0,3}|\*)?(?:,(?:[a-z0-9_]|%[0-9a-f]{2})+(?::[1-9][0-9]{0,3}|\*)?)*})*\z"#
+/// RFC 6570 Level 4 URI Template validator.
+///
+/// Single-pass parser with no allocations, early exit on invalid input.
+/// Supports all operators (+, #, ., /, ;, ?, &, =, ,, !, @, |) and modifiers (:prefix, *explode).
+fn is_valid_uri_template(template: &str) -> bool {
+    let bytes = template.as_bytes();
+    let len = bytes.len();
+    let mut i = 0;
+
+    while i < len {
+        if bytes[i] == b'{' {
+            // Parse expression
+            i += 1;
+            if i >= len {
+                return false; // Unclosed brace
+            }
+
+            // Optional operator
+            if is_operator(bytes[i]) {
+                i += 1;
+                if i >= len {
+                    return false;
+                }
+            }
+
+            // Parse variable list (at least one varspec required)
+            if !parse_varspec(bytes, &mut i) {
+                return false;
+            }
+
+            // Parse additional varspecs separated by commas
+            while i < len && bytes[i] == b',' {
+                i += 1;
+                if !parse_varspec(bytes, &mut i) {
+                    return false;
+                }
+            }
+
+            // Expect closing brace
+            if i >= len || bytes[i] != b'}' {
+                return false;
+            }
+            i += 1;
+        } else if bytes[i] == b'}' {
+            // Unmatched closing brace
+            return false;
+        } else {
+            // Parse literal
+            if !parse_literal(bytes, &mut i) {
+                return false;
+            }
+        }
+    }
+
+    true
+}
+
+/// Check if byte is an RFC 6570 operator (Appendix A grammar).
+/// Level 2: + (reserved expansion), # (fragment)
+/// Level 3: . (label), / (path), ; (path-style param), ? (query), & (query continuation)
+/// Reserved: = , ! @ | (reserved for future extensions per Section 2.2)
+#[inline]
+fn is_operator(b: u8) -> bool {
+    matches!(
+        b,
+        b'+' | b'#' | b'.' | b'/' | b';' | b'?' | b'&' | b'=' | b',' | b'!' | b'@' | b'|'
     )
-    .expect("Is a valid regex")
-});
+}
+
+/// Parse a variable specification: varname \[ modifier \]
+/// Returns false if invalid, updates index on success.
+#[inline]
+fn parse_varspec(bytes: &[u8], i: &mut usize) -> bool {
+    let len = bytes.len();
+
+    // Parse varname (required)
+    if !parse_varname(bytes, i) {
+        return false;
+    }
+
+    // Optional modifier
+    if *i < len {
+        match bytes[*i] {
+            b':' => {
+                // Prefix modifier: ":" max-length (1-9999)
+                *i += 1;
+                if *i >= len {
+                    return false;
+                }
+                // First digit must be 1-9
+                if !bytes[*i].is_ascii_digit() || bytes[*i] == b'0' {
+                    return false;
+                }
+                *i += 1;
+                // Up to 3 more digits (total max 4 digits for 1-9999)
+                let mut digit_count = 1;
+                while *i < len && bytes[*i].is_ascii_digit() && digit_count < 4 {
+                    *i += 1;
+                    digit_count += 1;
+                }
+            }
+            b'*' => {
+                // Explode modifier
+                *i += 1;
+            }
+            _ => {}
+        }
+    }
+
+    true
+}
+
+/// Parse a variable name: `varchar *( ["."] varchar )`
+/// varchar = ALPHA / DIGIT / "_" / pct-encoded
+#[inline]
+fn parse_varname(bytes: &[u8], i: &mut usize) -> bool {
+    let len = bytes.len();
+
+    // Must have at least one varchar
+    if !parse_varchar(bytes, i) {
+        return false;
+    }
+
+    // Continue parsing [ "." ] varchar
+    while *i < len {
+        if bytes[*i] == b'.' {
+            *i += 1;
+            if !parse_varchar(bytes, i) {
+                return false;
+            }
+        } else if is_varchar_start(bytes[*i]) || bytes[*i] == b'%' {
+            if !parse_varchar(bytes, i) {
+                return false;
+            }
+        } else {
+            break;
+        }
+    }
+
+    true
+}
+
+/// Parse one or more varchar characters.
+#[inline]
+fn parse_varchar(bytes: &[u8], i: &mut usize) -> bool {
+    let len = bytes.len();
+    let start = *i;
+
+    while *i < len {
+        if is_varchar_start(bytes[*i]) {
+            *i += 1;
+        } else if bytes[*i] == b'%' {
+            // pct-encoded
+            if *i + 2 >= len {
+                return false;
+            }
+            if !is_hex_digit(bytes[*i + 1]) || !is_hex_digit(bytes[*i + 2]) {
+                return false;
+            }
+            *i += 3;
+        } else {
+            break;
+        }
+    }
+
+    *i > start
+}
+
+/// Check if byte can start a varchar (ALPHA / DIGIT / "_").
+#[inline]
+fn is_varchar_start(b: u8) -> bool {
+    b.is_ascii_alphanumeric() || b == b'_'
+}
+
+/// Check if byte is a hexadecimal digit.
+#[inline]
+fn is_hex_digit(b: u8) -> bool {
+    b.is_ascii_hexdigit()
+}
+
+/// Parse a literal character or percent-encoded sequence.
+/// Returns false if invalid character found.
+#[inline]
+fn parse_literal(bytes: &[u8], i: &mut usize) -> bool {
+    let len = bytes.len();
+    let b = bytes[*i];
+
+    if b == b'%' {
+        // pct-encoded
+        if *i + 2 >= len {
+            return false;
+        }
+        if !is_hex_digit(bytes[*i + 1]) || !is_hex_digit(bytes[*i + 2]) {
+            return false;
+        }
+        *i += 3;
+        true
+    } else if is_literal_char(b) {
+        *i += 1;
+        true
+    } else {
+        false
+    }
+}
+
+/// Check if byte is a valid literal character per RFC 6570.
+/// Excludes: CTL (0x00-0x1F, 0x7F), space (0x20), and: `" ' < > % \ ^ { | }`
+#[inline]
+fn is_literal_char(b: u8) -> bool {
+    !matches!(
+        b,
+        0x00..=0x20
+            | b'"'
+            | b'\''
+            | b'<'
+            | b'>'
+            | b'%'
+            | b'\\'
+            | b'^'
+            | b'`'
+            | b'{'
+            | b'|'
+            | b'}'
+            | 0x7F
+    )
+}
 
 fn is_valid_json_pointer(pointer: &str) -> bool {
     if pointer.is_empty() {
@@ -149,27 +368,27 @@ fn is_leap_year(year: u16) -> bool {
 
 #[inline]
 fn parse_four_digits(bytes: &[u8]) -> Option<u16> {
-    let value = u32::from_ne_bytes([bytes[0], bytes[1], bytes[2], bytes[3]]);
-
+    // Little-endian layout: bytes[0] lands in the lowest byte of the u32.
     // Check if all bytes are ASCII digits
-    if value.wrapping_sub(0x3030_3030) & 0xF0F0_F0F0 == 0 {
-        let val = (value & 0x0F0F_0F0F).wrapping_mul(2561) >> 8;
-        Some(((val & 0x00FF_00FF).wrapping_mul(6_553_601) >> 16) as u16)
-    } else {
-        None
+    let value = u32::from_le_bytes([bytes[0], bytes[1], bytes[2], bytes[3]]);
+    let sub = value.wrapping_sub(0x3030_3030);
+    if sub.wrapping_add(0x0606_0606) & 0xF0F0_F0F0 != 0 {
+        return None;
     }
+    let val = (sub & 0x0F0F_0F0F).wrapping_mul(2561) >> 8;
+    Some(((val & 0x00FF_00FF).wrapping_mul(6_553_601) >> 16) as u16)
 }
 
 #[inline]
 fn parse_two_digits(bytes: &[u8]) -> Option<u8> {
-    let value = u16::from_ne_bytes([bytes[0], bytes[1]]);
-
-    // Check if both bytes are ASCII digits
-    if value.wrapping_sub(0x3030) & 0xF0F0 == 0 {
-        Some(((value & 0x0F0F).wrapping_mul(2561) >> 8) as u8)
-    } else {
-        None
+    // Little-endian layout: bytes[0] lands in the low byte of the u16.
+    // Check if all bytes are ASCII digits
+    let value = u16::from_le_bytes([bytes[0], bytes[1]]);
+    let sub = value.wrapping_sub(0x3030);
+    if sub.wrapping_add(0x0606) & 0xF0F0 != 0 {
+        return None;
     }
+    Some(((sub & 0x0F0F).wrapping_mul(2561) >> 8) as u8)
 }
 
 macro_rules! handle_offset {
@@ -691,32 +910,55 @@ fn is_valid_uri_reference(uri_reference: &str) -> bool {
     referencing::UriRef::parse(uri_reference).is_ok()
 }
 
-fn is_valid_regex(regex: &str) -> bool {
-    ecma::to_rust_regex(regex).is_ok()
-}
-
-fn is_valid_uri_template(uri_template: &str) -> bool {
-    URI_TEMPLATE_RE
-        .is_match(uri_template)
-        .expect("Simple URI_TEMPLATE_RE pattern")
-}
-
 fn is_valid_uuid(uuid: &str) -> bool {
     let mut out = [0; 16];
     parse_hyphenated(uuid.as_bytes(), Out::from_mut(&mut out)).is_ok()
 }
 
+/// Implements `evaluate()` for format validators that have an `annotation: Arc<Value>` field.
+///
+/// Per spec §7.2.1 and §7.2.2, the format value MUST be collected as an annotation
+/// regardless of whether the assertion passes or fails.
+macro_rules! impl_format_evaluate {
+    () => {
+        fn evaluate(
+            &self,
+            instance: &Value,
+            location: &LazyLocation,
+            tracker: Option<&RefTracker>,
+            ctx: &mut ValidationContext,
+        ) -> EvaluationResult {
+            if !instance.is_string() {
+                return EvaluationResult::valid_empty();
+            }
+            let errors: Vec<_> = self
+                .iter_errors(instance, location, tracker, ctx)
+                .map(|e| crate::evaluation::ErrorDescription::from_validation_error(&e))
+                .collect();
+            let mut result = if errors.is_empty() {
+                EvaluationResult::valid_empty()
+            } else {
+                EvaluationResult::invalid_empty(errors)
+            };
+            result.annotate(Annotations::from_arc(Arc::clone(&self.annotation)));
+            result
+        }
+    };
+}
+
 macro_rules! format_validators {
     ($(($validator:ident, $format:expr, $validation_fn:ident)),+ $(,)?) => {
         $(
             struct $validator {
                 location: Location,
+                annotation: Arc<Value>,
             }
 
             impl $validator {
                 pub(crate) fn compile<'a>(ctx: &compiler::Context) -> CompilationResult<'a> {
                     let location = ctx.location().join("format");
-                    Ok(Box::new($validator { location }))
+                    let annotation = Arc::new(Value::String($format.to_owned()));
+                    Ok(Box::new($validator { location, annotation }))
                 }
             }
 
@@ -749,6 +991,8 @@ macro_rules! format_validators {
                     }
                     Ok(())
                 }
+
+                impl_format_evaluate!();
             }
         )+
     };
@@ -768,7 +1012,6 @@ format_validators!(
         is_valid_iri_reference
     ),
     (JsonPointerValidator, "json-pointer", is_valid_json_pointer),
-    (RegexValidator, "regex", is_valid_regex),
     (
         RelativeJsonPointerValidator,
         "relative-json-pointer",
@@ -785,18 +1028,71 @@ format_validators!(
     (UuidValidator, "uuid", is_valid_uuid),
 );
 
+// Custom RegexValidator that caches ECMA regex transformation results in ValidationContext
+struct RegexValidator {
+    location: Location,
+    annotation: Arc<Value>,
+}
+
+impl RegexValidator {
+    pub(crate) fn compile<'a>(ctx: &compiler::Context) -> CompilationResult<'a> {
+        let location = ctx.location().join("format");
+        let annotation = Arc::new(Value::String("regex".to_owned()));
+        Ok(Box::new(RegexValidator {
+            location,
+            annotation,
+        }))
+    }
+}
+
+impl Validate for RegexValidator {
+    fn is_valid(&self, instance: &Value, ctx: &mut ValidationContext) -> bool {
+        if let Value::String(item) = instance {
+            ctx.is_valid_ecma_regex(item)
+        } else {
+            true
+        }
+    }
+
+    fn validate<'i>(
+        &self,
+        instance: &'i Value,
+        location: &LazyLocation,
+        tracker: Option<&RefTracker>,
+        ctx: &mut ValidationContext,
+    ) -> Result<(), ValidationError<'i>> {
+        if let Value::String(_) = instance {
+            if !self.is_valid(instance, ctx) {
+                return Err(ValidationError::format(
+                    self.location.clone(),
+                    crate::paths::capture_evaluation_path(tracker, &self.location),
+                    location.into(),
+                    instance,
+                    "regex",
+                ));
+            }
+        }
+        Ok(())
+    }
+
+    impl_format_evaluate!();
+}
+
 // Custom EmailValidator that supports email options
 struct EmailValidator {
     location: Location,
+    annotation: Arc<Value>,
     email_options: Option<EmailAddressOptions>,
 }
 
 impl EmailValidator {
     pub(crate) fn compile<'a>(ctx: &compiler::Context) -> CompilationResult<'a> {
         let location = ctx.location().join("format");
+        let annotation = Arc::new(Value::String("email".to_owned()));
         let email_options = ctx.config().email_options().copied();
         Ok(Box::new(EmailValidator {
             location,
+            annotation,
             email_options,
         }))
     }
@@ -831,20 +1127,25 @@ impl Validate for EmailValidator {
         }
         Ok(())
     }
+
+    impl_format_evaluate!();
 }
 
 // Custom IdnEmailValidator that supports email options
 struct IdnEmailValidator {
     location: Location,
+    annotation: Arc<Value>,
     email_options: Option<EmailAddressOptions>,
 }
 
 impl IdnEmailValidator {
     pub(crate) fn compile<'a>(ctx: &compiler::Context) -> CompilationResult<'a> {
         let location = ctx.location().join("format");
+        let annotation = Arc::new(Value::String("idn-email".to_owned()));
         let email_options = ctx.config().email_options().copied();
         Ok(Box::new(IdnEmailValidator {
             location,
+            annotation,
             email_options,
         }))
     }
@@ -879,10 +1180,13 @@ impl Validate for IdnEmailValidator {
         }
         Ok(())
     }
+
+    impl_format_evaluate!();
 }
 
 struct CustomFormatValidator {
     location: Location,
+    annotation: Arc<Value>,
     format_name: String,
     check: Arc<dyn Format>,
 }
@@ -893,8 +1197,10 @@ impl CustomFormatValidator {
         check: Arc<dyn Format>,
     ) -> CompilationResult<'a> {
         let location = ctx.location().join("format");
+        let annotation = Arc::new(Value::String(format_name.clone()));
         Ok(Box::new(CustomFormatValidator {
             location,
+            annotation,
             format_name,
             check,
         }))
@@ -929,6 +1235,56 @@ impl Validate for CustomFormatValidator {
             true
         }
     }
+
+    impl_format_evaluate!();
+}
+
+/// Format annotation-only validator used when format assertion is disabled.
+///
+/// Always validates successfully but emits the required annotation per spec §7.2.1.
+struct AnnotationOnlyFormatValidator {
+    annotation: Arc<Value>,
+}
+
+impl Validate for AnnotationOnlyFormatValidator {
+    fn is_valid(&self, _instance: &Value, _ctx: &mut ValidationContext) -> bool {
+        true
+    }
+
+    fn validate<'i>(
+        &self,
+        _instance: &'i Value,
+        _location: &LazyLocation,
+        _tracker: Option<&RefTracker>,
+        _ctx: &mut ValidationContext,
+    ) -> Result<(), ValidationError<'i>> {
+        Ok(())
+    }
+
+    fn iter_errors<'i>(
+        &self,
+        _instance: &'i Value,
+        _location: &LazyLocation,
+        _tracker: Option<&RefTracker>,
+        _ctx: &mut ValidationContext,
+    ) -> crate::error::ErrorIterator<'i> {
+        no_error()
+    }
+
+    fn evaluate(
+        &self,
+        instance: &Value,
+        _location: &LazyLocation,
+        _tracker: Option<&RefTracker>,
+        _ctx: &mut ValidationContext,
+    ) -> EvaluationResult {
+        if !instance.is_string() {
+            return EvaluationResult::valid_empty();
+        }
+        let mut result = EvaluationResult::valid_empty();
+        result.annotate(Annotations::from_arc(Arc::clone(&self.annotation)));
+        result
+    }
 }
 
 pub(crate) trait Format: Send + Sync + 'static {
@@ -951,55 +1307,71 @@ pub(crate) fn compile<'a>(
     _: &'a Map<String, Value>,
     schema: &'a Value,
 ) -> Option<CompilationResult<'a>> {
-    if !ctx.validates_formats_by_default() {
-        return None;
-    }
-
     if let Value::String(format) = schema {
-        if let Some((name, func)) = ctx.get_format(format) {
-            return Some(CustomFormatValidator::compile(
-                ctx,
-                name.clone(),
-                func.clone(),
-            ));
-        }
-        let draft = ctx.draft();
-        match format.as_str() {
-            "date" => Some(DateValidator::compile(ctx)),
-            "date-time" => Some(DateTimeValidator::compile(ctx)),
-            "duration" if draft >= Draft::Draft201909 => Some(DurationValidator::compile(ctx)),
-            "email" => Some(EmailValidator::compile(ctx)),
-            "hostname" => Some(HostnameValidator::compile(ctx)),
-            "idn-email" => Some(IdnEmailValidator::compile(ctx)),
-            "idn-hostname" if draft >= Draft::Draft7 => Some(IdnHostnameValidator::compile(ctx)),
-            "ipv4" => Some(IpV4Validator::compile(ctx)),
-            "ipv6" => Some(IpV6Validator::compile(ctx)),
-            "iri" if draft >= Draft::Draft7 => Some(IriValidator::compile(ctx)),
-            "iri-reference" if draft >= Draft::Draft7 => Some(IriReferenceValidator::compile(ctx)),
-            "json-pointer" if draft >= Draft::Draft6 => Some(JsonPointerValidator::compile(ctx)),
-            "regex" => Some(RegexValidator::compile(ctx)),
-            "relative-json-pointer" if draft >= Draft::Draft7 => {
-                Some(RelativeJsonPointerValidator::compile(ctx))
+        if ctx.validates_formats_by_default() {
+            // Format validation is enabled: each specific validator carries its own annotation
+            if let Some((name, func)) = ctx.get_format(format) {
+                return Some(CustomFormatValidator::compile(
+                    ctx,
+                    name.clone(),
+                    func.clone(),
+                ));
             }
-            "time" => Some(TimeValidator::compile(ctx)),
-            "uri" => Some(UriValidator::compile(ctx)),
-            "uri-reference" if draft >= Draft::Draft6 => Some(UriReferenceValidator::compile(ctx)),
-            "uri-template" if draft >= Draft::Draft6 => Some(UriTemplateValidator::compile(ctx)),
-            "uuid" if draft >= Draft::Draft201909 => Some(UuidValidator::compile(ctx)),
-            name => {
-                if ctx.are_unknown_formats_ignored() {
-                    None
-                } else {
-                    let location = ctx.location().join("format");
-                    Some(Err(ValidationError::compile_error(
-                        location.clone(),
-                        location,
-                        Location::new(),
-                        schema,
-                        format!("Unknown format: '{name}'. Adjust configuration to ignore unrecognized formats"),
-                    )))
+            let draft = ctx.draft();
+            match format.as_str() {
+                "date" => Some(DateValidator::compile(ctx)),
+                "date-time" => Some(DateTimeValidator::compile(ctx)),
+                "duration" if draft >= Draft::Draft201909 => Some(DurationValidator::compile(ctx)),
+                "email" => Some(EmailValidator::compile(ctx)),
+                "hostname" => Some(HostnameValidator::compile(ctx)),
+                "idn-email" => Some(IdnEmailValidator::compile(ctx)),
+                "idn-hostname" if draft >= Draft::Draft7 => {
+                    Some(IdnHostnameValidator::compile(ctx))
+                }
+                "ipv4" => Some(IpV4Validator::compile(ctx)),
+                "ipv6" => Some(IpV6Validator::compile(ctx)),
+                "iri" if draft >= Draft::Draft7 => Some(IriValidator::compile(ctx)),
+                "iri-reference" if draft >= Draft::Draft7 => {
+                    Some(IriReferenceValidator::compile(ctx))
+                }
+                "json-pointer" if draft >= Draft::Draft6 => {
+                    Some(JsonPointerValidator::compile(ctx))
+                }
+                "regex" => Some(RegexValidator::compile(ctx)),
+                "relative-json-pointer" if draft >= Draft::Draft7 => {
+                    Some(RelativeJsonPointerValidator::compile(ctx))
+                }
+                "time" => Some(TimeValidator::compile(ctx)),
+                "uri" => Some(UriValidator::compile(ctx)),
+                "uri-reference" if draft >= Draft::Draft6 => {
+                    Some(UriReferenceValidator::compile(ctx))
+                }
+                "uri-template" if draft >= Draft::Draft6 => {
+                    Some(UriTemplateValidator::compile(ctx))
+                }
+                "uuid" if draft >= Draft::Draft201909 => Some(UuidValidator::compile(ctx)),
+                name => {
+                    if ctx.are_unknown_formats_ignored() {
+                        None
+                    } else {
+                        let location = ctx.location().join("format");
+                        Some(Err(ValidationError::compile_error(
+                            location.clone(),
+                            location,
+                            Location::new(),
+                            schema,
+                            format!(
+                                "Unknown format: '{name}'. Adjust configuration to ignore unrecognized formats"
+                            ),
+                        )))
+                    }
                 }
             }
+        } else {
+            // Format validation disabled: annotation-only per spec §7.2.1
+            Some(Ok(Box::new(AnnotationOnlyFormatValidator {
+                annotation: Arc::new(Value::String(format.clone())),
+            })))
         }
     } else {
         let location = ctx.location().join("format");
@@ -1023,6 +1395,30 @@ mod tests {
 
     use super::*;
 
+    #[test_case(b"00" => Some(0);  "min")]
+    #[test_case(b"09" => Some(9);  "nine")]
+    #[test_case(b"10" => Some(10); "ten")]
+    #[test_case(b"59" => Some(59); "fifty-nine")]
+    #[test_case(b"99" => Some(99); "max")]
+    #[test_case(b"1a" => None; "alpha")]
+    #[test_case(b" 5" => None; "leading space")]
+    #[test_case(b":0" => None; "colon is not a digit")]
+    #[test_case(b";9" => None; "semicolon is not a digit")]
+    fn test_parse_two_digits(bytes: &[u8]) -> Option<u8> {
+        parse_two_digits(bytes)
+    }
+
+    #[test_case(b"0000" => Some(0);    "zero")]
+    #[test_case(b"1970" => Some(1970); "epoch year")]
+    #[test_case(b"2023" => Some(2023); "recent year")]
+    #[test_case(b"9999" => Some(9999); "max")]
+    #[test_case(b"199x" => None; "trailing alpha")]
+    #[test_case(b" 999" => None; "leading space")]
+    #[test_case(b"20:3" => None; "colon is not a digit")]
+    fn test_parse_four_digits(bytes: &[u8]) -> Option<u16> {
+        parse_four_digits(bytes)
+    }
+
     #[test]
     fn ignored_format() {
         let schema = json!({"format": "custom", "type": "string"});
@@ -1141,6 +1537,8 @@ mod tests {
     #[test_case("aaaa-01-12", false; "Malformed (letters in year)")]
     #[test_case("2000-bb-12", false; "Malformed (letters in month)")]
     #[test_case("2000-01-cc", false; "Malformed (letters in day)")]
+    #[test_case("20:3-01-15", false; "colon in year")]
+    #[test_case("20;3-01-15", false; "semicolon in year")]
     fn test_is_valid_date(input: &str, expected: bool) {
         assert_eq!(is_valid_date(input), expected);
     }
@@ -1489,4 +1887,140 @@ mod tests {
         // Should accept valid email with TLD
         assert!(validator.is_valid(&json!("user@example.com")));
     }
+
+    // Simple valid templates
+    #[test_case(""; "empty string")]
+    #[test_case("http://example.com/"; "plain URL")]
+    #[test_case("no-template-here"; "no templates")]
+    #[test_case("{var}"; "simple variable")]
+    #[test_case("http://example.com/{var}"; "URL with variable")]
+    #[test_case("http://example.com/dictionary/{term}"; "URL with named variable")]
+    #[test_case("/users/{id}"; "path with variable")]
+    #[test_case("http://example.com/~{username}/"; "URL with tilde")]
+    // All operators
+    #[test_case("{+var}"; "reserved expansion")]
+    #[test_case("{#var}"; "fragment expansion")]
+    #[test_case("{.var}"; "label expansion with dot")]
+    #[test_case("{/var}"; "path segment expansion")]
+    #[test_case("{;var}"; "path-style parameter expansion")]
+    #[test_case("{?var}"; "query expansion")]
+    #[test_case("{&var}"; "query continuation expansion")]
+    // RFC 6570 reserved operators (Section 2.2, Appendix A) - syntactically valid
+    #[test_case("{=var}"; "reserved operator equals")]
+    #[test_case("{,var}"; "reserved operator comma")]
+    #[test_case("{!var}"; "reserved operator exclamation")]
+    #[test_case("{@var}"; "reserved operator at")]
+    #[test_case("{|var}"; "reserved operator pipe")]
+    #[test_case("{|var*}"; "reserved operator with explode")]
+    // Prefix modifier valid on any variable (RFC 6570 Section 2.4.1)
+    #[test_case("{keys:1}"; "prefix modifier on any var")]
+    #[test_case("{+keys:1}"; "operator with prefix modifier")]
+    // Modifiers
+    #[test_case("{var:10}"; "prefix modifier")]
+    #[test_case("{var:1}"; "prefix modifier min")]
+    #[test_case("{var:9999}"; "prefix modifier max")]
+    #[test_case("{var*}"; "explode modifier")]
+    #[test_case("{+var*}"; "operator with explode")]
+    #[test_case("{#var:5}"; "operator with prefix")]
+    // Multiple variables
+    #[test_case("{var1,var2}"; "multiple variables")]
+    #[test_case("{var1,var2,var3}"; "three variables")]
+    #[test_case("{+var1,var2}"; "operator with multiple variables")]
+    #[test_case("{var1:5,var2*}"; "multiple variables with modifiers")]
+    // Complex templates
+    #[test_case("http://example.com{+path}{?query*}"; "complex template")]
+    #[test_case("http://example.com{#fragment}"; "fragment template")]
+    #[test_case("http://example.com{.dom*}"; "domain template")]
+    #[test_case("http://example.com{/path,path2}"; "path template")]
+    #[test_case("http://example.com{;params*}"; "params template")]
+    #[test_case("http://example.com{?query,query2}"; "query template")]
+    #[test_case("http://example.com/{var1}/{var2}/{var3}"; "multiple expressions")]
+    // Variable names with dots
+    #[test_case("{var.name}"; "dotted variable name")]
+    #[test_case("{a.b.c}"; "multiple dots in variable name")]
+    // Percent-encoded in variable names
+    #[test_case("{%20}"; "percent-encoded space in varname")]
+    #[test_case("{a%20b}"; "percent-encoded in middle of varname")]
+    #[test_case("{%41}"; "percent-encoded A")]
+    // Percent-encoded in literals
+    #[test_case("http://example.com/%20space"; "percent-encoded in URL")]
+    #[test_case("hello%20world"; "percent-encoded space")]
+    fn test_valid_uri_template(template: &str) {
+        assert!(is_valid_uri_template(template));
+    }
+
+    // Invalid templates
+    #[test_case("{"; "unclosed brace")]
+    #[test_case("}"; "unmatched close brace")]
+    #[test_case("{}"; "empty expression")]
+    #[test_case("{+}"; "operator only")]
+    #[test_case("http://example.com/{unclosed"; "unclosed in URL")]
+    #[test_case("http://example.com/{var"; "missing close brace")]
+    #[test_case("http://example.com/}"; "extra close brace")]
+    // Invalid modifiers
+    #[test_case("{var:0}"; "prefix zero")]
+    #[test_case("{var:}"; "prefix empty")]
+    #[test_case("{var:10000}"; "prefix too large")]
+    #[test_case("{var::5}"; "double colon")]
+    #[test_case("{var**}"; "double explode")]
+    #[test_case("{*}"; "explode only")]
+    #[test_case("{:5}"; "prefix only")]
+    // Invalid variable names
+    #[test_case("{-var}"; "hyphen start")]
+    #[test_case("{var-}"; "hyphen in variable")]
+    #[test_case("{.}"; "dot only")]
+    #[test_case("{..var}"; "double dot start")]
+    #[test_case("{var..name}"; "double dot in name")]
+    // Invalid percent encoding
+    #[test_case("{%}"; "incomplete percent")]
+    #[test_case("{%Z}"; "incomplete percent hex")]
+    #[test_case("{%ZZ}"; "invalid hex digits")]
+    #[test_case("{%0}"; "single hex digit")]
+    #[test_case("%"; "incomplete percent in literal")]
+    #[test_case("%Z"; "incomplete percent in literal 2")]
+    #[test_case("%ZZ"; "invalid hex in literal")]
+    // Invalid characters in literals
+    #[test_case("hello world"; "space in literal")]
+    #[test_case("hello\ttab"; "tab in literal")]
+    #[test_case("hello\nline"; "newline in literal")]
+    #[test_case("hello\"quote"; "quote in literal")]
+    #[test_case("hello<angle"; "angle bracket in literal")]
+    #[test_case("hello\\back"; "backslash in literal")]
+    #[test_case("hello^caret"; "caret in literal")]
+    #[test_case("hello`backtick"; "backtick in literal")]
+    #[test_case("hello|pipe"; "pipe in literal")]
+    // Invalid expressions
+    #[test_case("{var,}"; "trailing comma")]
+    #[test_case("{var,,var2}"; "double comma")]
+    // uritemplate-test/negative-tests.json
+    // Note: Per RFC 6570, reserved operators (=, ,, !, @, |) are syntactically valid.
+    // Tests 7, 11, 13, 21, 22 are valid per RFC syntax but fail in uritemplate-test
+    // because they test expansion behavior, not syntax validation.
+    #[test_case("{/id*"; "uritemplate-test 1: unclosed brace")]
+    #[test_case("/id*}"; "uritemplate-test 2: unmatched close brace")]
+    #[test_case("{/?id}"; "uritemplate-test 3: question mark in varname")]
+    #[test_case("{var:prefix}"; "uritemplate-test 4: non-numeric prefix")]
+    #[test_case("{hello:2*}"; "uritemplate-test 5: prefix and explode combined")]
+    #[test_case("{??hello}"; "uritemplate-test 6: question mark in varname")]
+    #[test_case("{with space}"; "uritemplate-test 8: space in variable name")]
+    #[test_case("{ leading_space}"; "uritemplate-test 9: leading space in expression")]
+    #[test_case("{trailing_space }"; "uritemplate-test 10: trailing space in expression")]
+    #[test_case("{$var}"; "uritemplate-test 12: dollar sign not a valid operator")]
+    #[test_case("{*keys?}"; "uritemplate-test 14: explode at start of varspec")]
+    #[test_case("{?empty=default,var}"; "uritemplate-test 15: equals sign in varname")]
+    #[test_case("{var}{-prefix|/-/|var}"; "uritemplate-test 16: hyphen not a valid operator")]
+    #[test_case("?q={searchTerms}&amp;c={example:color?}"; "uritemplate-test 17: non-numeric prefix")]
+    #[test_case("x{?empty|foo=none}"; "uritemplate-test 18: invalid chars after varname")]
+    #[test_case("/h{#hello+}"; "uritemplate-test 19: plus in varname")]
+    #[test_case("/h#{hello+}"; "uritemplate-test 20: plus in varname")]
+    #[test_case("{;keys:1*}"; "uritemplate-test 23: prefix and explode combined")]
+    #[test_case("?{-join|&|var,list}"; "uritemplate-test 24: hyphen not a valid operator")]
+    #[test_case("{~thing}"; "uritemplate-test 25: tilde not a valid operator")]
+    #[test_case("/{default-graph-uri}"; "uritemplate-test 26: hyphen in varname")]
+    #[test_case("/sparql{?query,default-graph-uri}"; "uritemplate-test 27: hyphen in varname")]
+    #[test_case("/sparql{?query){&default-graph-uri*}"; "uritemplate-test 28: paren in template")]
+    #[test_case("/resolution{?x, y}"; "uritemplate-test 29: space after comma")]
+    fn test_invalid_uri_template(template: &str) {
+        assert!(!is_valid_uri_template(template));
+    }
 }
