@@ -30,8 +30,8 @@ pub(crate) type PendingPropertyValidators = Arc<OnceLock<PropertyValidators>>;
 /// This structure is built during schema compilation and used during validation.
 #[derive(Debug, Clone)]
 pub(crate) struct PropertyValidators {
-    /// Direct property validators from "properties" keyword
-    properties: Vec<(String, SchemaNode)>,
+    /// Property names from "properties" keyword for O(1) lookup
+    properties: AHashSet<String>,
     /// Validator from "additionalProperties" keyword
     additional: Option<SchemaNode>,
     /// Pattern-based property validators from "patternProperties" keyword
@@ -72,13 +72,18 @@ struct ConditionalValidators {
 }
 
 impl PropertyValidators {
-    /// Mark all properties that are evaluated by this schema.
-    /// This is the core logic for determining which properties should not be considered "unevaluated".
-    fn mark_evaluated_properties<'i>(
+    /// Core implementation for marking evaluated properties.
+    ///
+    /// When `include_unevaluated` is `true` (used by `is_valid`/`validate`), also marks
+    /// properties validated by `unevaluatedProperties` itself — needed so nested schemas
+    /// can propagate evaluations upward. When `false` (used by `evaluate`), those properties
+    /// are left unmarked so `evaluate_instance()` is called on them to collect annotations.
+    fn mark_evaluated_properties_impl<'i>(
         &self,
         instance: &'i Value,
         properties: &mut AHashSet<&'i String>,
         ctx: &mut ValidationContext,
+        include_unevaluated: bool,
     ) {
         // Handle $ref first
         if let Some(ref_) = &self.ref_ {
@@ -100,9 +105,9 @@ impl PropertyValidators {
 
         // Process properties on the instance
         if let Value::Object(obj) = instance {
-            // Mark properties from "properties" keyword
+            // Mark properties from "properties" keyword (O(1) lookup)
             for property in obj.keys() {
-                if self.properties.iter().any(|(p, _)| p == property) {
+                if self.properties.contains(property) {
                     properties.insert(property);
                 }
             }
@@ -133,17 +138,20 @@ impl PropertyValidators {
                 }
             }
 
-            // Check "unevaluatedProperties" keyword - marks properties that validate successfully
+            // Check "unevaluatedProperties" keyword - marks properties that validate successfully.
             // This is crucial for nested unevaluatedProperties: a child schema's unevaluatedProperties
-            // can mark properties as evaluated for parent schemas
-            if let Some(unevaluated) = &self.unevaluated {
-                for (property, value) in obj {
-                    // Skip if already marked - avoid redundant validation
-                    if properties.contains(property) {
-                        continue;
-                    }
-                    if unevaluated.is_valid(value, ctx) {
-                        properties.insert(property);
+            // can mark properties as evaluated for parent schemas.
+            // Skipped when called from evaluate() so evaluate_instance() can collect annotations.
+            if include_unevaluated {
+                if let Some(unevaluated) = &self.unevaluated {
+                    for (property, value) in obj {
+                        // Skip if already marked - avoid redundant validation
+                        if properties.contains(property) {
+                            continue;
+                        }
+                        if unevaluated.is_valid(value, ctx) {
+                            properties.insert(property);
+                        }
                     }
                 }
             }
@@ -176,20 +184,46 @@ impl PropertyValidators {
         }
 
         // Handle "oneOf" keyword - only if exactly one matches
-        let one_of_matches: Vec<bool> = self
-            .one_of
-            .iter()
-            .map(|(node, _)| node.is_valid(instance, ctx))
-            .collect();
-
-        if one_of_matches.iter().filter(|&&v| v).count() == 1 {
-            for ((_node, validators), &is_valid) in self.one_of.iter().zip(one_of_matches.iter()) {
-                if is_valid {
-                    validators.mark_evaluated_properties(instance, properties, ctx);
-                    break;
+        // Short-circuit: stop checking after finding 2 matches
+        let mut match_count = 0;
+        let mut matched_validators = None;
+        for (node, validators) in &self.one_of {
+            if node.is_valid(instance, ctx) {
+                match_count += 1;
+                if match_count > 1 {
+                    break; // More than one match, don't mark any properties
                 }
+                matched_validators = Some(validators);
             }
         }
+        if match_count == 1 {
+            if let Some(validators) = matched_validators {
+                validators.mark_evaluated_properties(instance, properties, ctx);
+            }
+        }
+    }
+
+    /// Mark all properties evaluated by this schema (including by `unevaluatedProperties` itself).
+    fn mark_evaluated_properties<'i>(
+        &self,
+        instance: &'i Value,
+        properties: &mut AHashSet<&'i String>,
+        ctx: &mut ValidationContext,
+    ) {
+        self.mark_evaluated_properties_impl(instance, properties, ctx, true);
+    }
+
+    /// Mark properties evaluated by all keywords *except* `unevaluatedProperties` itself.
+    ///
+    /// Used in `evaluate()` so that properties that would be covered by `unevaluatedProperties`
+    /// are still visited by `evaluate_instance()`, allowing their annotations to be collected.
+    fn mark_evaluated_by_other_keywords<'i>(
+        &self,
+        instance: &'i Value,
+        properties: &mut AHashSet<&'i String>,
+        ctx: &mut ValidationContext,
+    ) {
+        self.mark_evaluated_properties_impl(instance, properties, ctx, false);
     }
 }
 
@@ -256,24 +290,14 @@ fn compile_property_validators<'a>(
 }
 
 fn compile_properties<'a>(
-    ctx: &compiler::Context<'_>,
+    _ctx: &compiler::Context<'_>,
     parent: &'a Map<String, Value>,
-) -> Result<Vec<(String, SchemaNode)>, ValidationError<'a>> {
+) -> Result<AHashSet<String>, ValidationError<'a>> {
     let Some(Value::Object(map)) = parent.get("properties") else {
-        return Ok(Vec::new());
+        return Ok(AHashSet::new());
     };
-
-    let properties_ctx = ctx.new_at_location("properties");
-    let mut result = Vec::with_capacity(map.len());
-
-    for (property, subschema) in map {
-        let prop_ctx = properties_ctx.new_at_location(property.as_str());
-        let node = compiler::compile(&prop_ctx, prop_ctx.as_resource_ref(subschema))
-            .map_err(ValidationError::to_owned)?;
-        result.push((property.clone(), node));
-    }
-
-    Ok(result)
+    // Only need property names for evaluation tracking, not the validators
+    Ok(map.keys().cloned().collect())
 }
 
 fn compile_additional<'a>(
@@ -666,7 +690,7 @@ impl Validate for UnevaluatedPropertiesValidator {
         if let Value::Object(properties) = instance {
             let mut evaluated = AHashSet::with_capacity(properties.len());
             self.validators
-                .mark_evaluated_properties(instance, &mut evaluated, ctx);
+                .mark_evaluated_by_other_keywords(instance, &mut evaluated, ctx);
             let mut children = Vec::new();
             let mut unevaluated = Vec::new();
             let mut invalid = false;
