@@ -20,6 +20,7 @@ pub(crate) struct SchemaNode {
     validators: Arc<NodeValidators>,
     location: Location,
     absolute_path: Option<Arc<Uri<String>>>,
+    formatted_schema_location: Arc<OnceLock<Arc<str>>>,
 }
 
 // Separate type used only during compilation for handling recursive references
@@ -28,11 +29,22 @@ pub(crate) struct PendingSchemaNode {
     cell: Arc<OnceLock<PendingTarget>>,
 }
 
-#[derive(Debug)]
 struct PendingTarget {
     validators: Weak<NodeValidators>,
     location: Location,
     absolute_path: Option<Arc<Uri<String>>>,
+    formatted_schema_location: Arc<OnceLock<Arc<str>>>,
+    /// Cached materialized `SchemaNode` to avoid cloning on every access.
+    cached_node: OnceLock<SchemaNode>,
+}
+
+impl fmt::Debug for PendingTarget {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("PendingTarget")
+            .field("location", &self.location)
+            .field("absolute_path", &self.absolute_path)
+            .finish_non_exhaustive()
+    }
 }
 
 enum NodeValidators {
@@ -79,12 +91,14 @@ struct KeywordValidatorEntry {
     validator: BoxedValidator,
     location: Location,
     absolute_location: Option<Arc<Uri<String>>>,
+    formatted_schema_location: OnceLock<Arc<str>>,
 }
 
 struct ArrayValidatorEntry {
     validator: BoxedValidator,
     location: Location,
     absolute_location: Option<Arc<Uri<String>>>,
+    formatted_schema_location: OnceLock<Arc<str>>,
 }
 
 impl PendingSchemaNode {
@@ -99,26 +113,27 @@ impl PendingSchemaNode {
             validators: Arc::downgrade(&node.validators),
             location: node.location.clone(),
             absolute_path: node.absolute_path.clone(),
+            formatted_schema_location: node.formatted_schema_location.clone(),
+            cached_node: OnceLock::new(),
         };
         self.cell
             .set(target)
             .expect("pending node initialized twice");
     }
 
-    pub(crate) fn get(&self) -> Option<SchemaNode> {
-        self.cell.get().map(PendingTarget::materialize)
+    pub(crate) fn get(&self) -> Option<&SchemaNode> {
+        self.cell.get().map(PendingTarget::get_or_materialize)
     }
 
     fn with_node<F, R>(&self, f: F) -> R
     where
-        F: FnOnce(SchemaNode) -> R,
+        F: FnOnce(&SchemaNode) -> R,
     {
-        let node = self
+        let target = self
             .cell
             .get()
-            .expect("pending node accessed before initialization")
-            .materialize();
-        f(node)
+            .expect("pending node accessed before initialization");
+        f(target.get_or_materialize())
     }
 
     /// Get a unique identifier for this pending node.
@@ -130,16 +145,20 @@ impl PendingSchemaNode {
 }
 
 impl PendingTarget {
-    fn materialize(&self) -> SchemaNode {
-        let validators = self
-            .validators
-            .upgrade()
-            .expect("pending schema target dropped");
-        SchemaNode {
-            validators,
-            location: self.location.clone(),
-            absolute_path: self.absolute_path.clone(),
-        }
+    /// Get or create the cached `SchemaNode`. Only clones on first access.
+    fn get_or_materialize(&self) -> &SchemaNode {
+        self.cached_node.get_or_init(|| {
+            let validators = self
+                .validators
+                .upgrade()
+                .expect("pending schema target dropped");
+            SchemaNode {
+                validators,
+                location: self.location.clone(),
+                absolute_path: self.absolute_path.clone(),
+                formatted_schema_location: self.formatted_schema_location.clone(),
+            }
+        })
     }
 }
 
@@ -208,18 +227,27 @@ impl Validate for PendingSchemaNode {
 
 impl SchemaNode {
     pub(crate) fn from_boolean(ctx: &Context<'_>, validator: Option<BoxedValidator>) -> SchemaNode {
+        let location = ctx.location().clone();
+        let absolute_path = ctx.base_uri();
         SchemaNode {
-            location: ctx.location().clone(),
-            absolute_path: ctx.base_uri(),
+            location,
+            absolute_path,
+            formatted_schema_location: Arc::new(OnceLock::new()),
             validators: Arc::new(NodeValidators::Boolean { validator }),
         }
     }
 
     pub(crate) fn from_keywords(
         ctx: &Context<'_>,
-        validators: Vec<(Keyword, BoxedValidator)>,
+        mut validators: Vec<(Keyword, BoxedValidator)>,
         unmatched_keywords: Option<Arc<Value>>,
     ) -> SchemaNode {
+        // Sort validators by priority (lower = execute first).
+        // This enables "fail fast" by running cheap validators (type, const)
+        // before expensive ones (allOf, $ref).
+        validators.sort_by_key(|(keyword, _)| crate::keywords::keyword_priority(keyword));
+
+        let location = ctx.location().clone();
         let absolute_path = ctx.base_uri();
         let validators = validators
             .into_iter()
@@ -230,12 +258,14 @@ impl SchemaNode {
                     validator,
                     location,
                     absolute_location,
+                    formatted_schema_location: OnceLock::new(),
                 }
             })
             .collect();
         SchemaNode {
-            location: ctx.location().clone(),
+            location,
             absolute_path,
+            formatted_schema_location: Arc::new(OnceLock::new()),
             validators: Arc::new(NodeValidators::Keyword(KeywordValidators {
                 unmatched_keywords,
                 validators,
@@ -244,6 +274,7 @@ impl SchemaNode {
     }
 
     pub(crate) fn from_array(ctx: &Context<'_>, validators: Vec<BoxedValidator>) -> SchemaNode {
+        let location = ctx.location().clone();
         let absolute_path = ctx.base_uri();
         let validators = validators
             .into_iter()
@@ -255,12 +286,14 @@ impl SchemaNode {
                     validator,
                     location,
                     absolute_location,
+                    formatted_schema_location: OnceLock::new(),
                 }
             })
             .collect();
         SchemaNode {
-            location: ctx.location().clone(),
+            location,
             absolute_path,
+            formatted_schema_location: Arc::new(OnceLock::new()),
             validators: Arc::new(NodeValidators::Array { validators }),
         }
     }
@@ -293,9 +326,9 @@ impl SchemaNode {
         let instance_location: Location = location.into();
 
         let keyword_location = crate::paths::evaluation_path(tracker, &self.location);
-
-        let schema_location =
-            crate::evaluation::format_schema_location(&self.location, self.absolute_path.as_ref());
+        let schema_location = Arc::clone(self.formatted_schema_location.get_or_init(|| {
+            crate::evaluation::format_schema_location(&self.location, self.absolute_path.as_ref())
+        }));
 
         match self.evaluate(instance, location, tracker, ctx) {
             EvaluationResult::Valid {
@@ -304,7 +337,7 @@ impl SchemaNode {
             } => EvaluationNode::valid(
                 keyword_location,
                 self.absolute_path.clone(),
-                schema_location,
+                schema_location.clone(),
                 instance_location,
                 annotations,
                 children,
@@ -339,6 +372,7 @@ impl SchemaNode {
                 Item = (
                     &'a Location,
                     Option<&'a Arc<Uri<String>>>,
+                    &'a OnceLock<Arc<str>>,
                     &'a BoxedValidator,
                 ),
             > + 'a,
@@ -349,7 +383,7 @@ impl SchemaNode {
 
         let instance_loc: Location = location.into();
 
-        for (child_location, absolute_location, validator) in subschemas {
+        for (child_location, absolute_location, cached_schema_location, validator) in subschemas {
             let child_result = validator.evaluate(instance, location, tracker, ctx);
 
             let absolute_location = absolute_location.cloned();
@@ -360,11 +394,20 @@ impl SchemaNode {
             // Per JSON Schema spec: "MUST NOT include by-reference applicators such as $ref"
             // For by-reference validators like $ref, use the target's canonical location.
             // For regular validators, use the keyword's location.
-            let schema_location = validator.canonical_location().unwrap_or(child_location);
-            let formatted_schema_location = crate::evaluation::format_schema_location(
-                schema_location,
-                absolute_location.as_ref(),
-            );
+            let formatted_schema_location =
+                if let Some(schema_location) = validator.canonical_location() {
+                    crate::evaluation::format_schema_location(
+                        schema_location,
+                        absolute_location.as_ref(),
+                    )
+                } else {
+                    Arc::clone(cached_schema_location.get_or_init(|| {
+                        crate::evaluation::format_schema_location(
+                            child_location,
+                            absolute_location.as_ref(),
+                        )
+                    }))
+                };
 
             let child_node = match child_result {
                 EvaluationResult::Valid {
@@ -533,6 +576,7 @@ impl Validate for SchemaNode {
                     (
                         &entry.location,
                         entry.absolute_location.as_ref(),
+                        &entry.formatted_schema_location,
                         &entry.validator,
                     )
                 }),
@@ -565,6 +609,7 @@ impl Validate for SchemaNode {
                         (
                             &entry.location,
                             entry.absolute_location.as_ref(),
+                            &entry.formatted_schema_location,
                             &entry.validator,
                         )
                     }),
